@@ -3,13 +3,14 @@
 
 from operator import itemgetter
 import urllib
-import time
 
 import click
 import dateutil.parser
+import os
 import pyalpm
 import pycman.pkginfo
 import requests
+import sys
 
 # monkeypatch pycman
 pycman.pkginfo.ATTRNAME_FORMAT = '%-13s : '
@@ -200,6 +201,10 @@ def format_signoff_short(signoff_pkg, local_pkg, options):
     elif options.show_uninstalled and local_pkg:
         formatted += click.style(" (installed)", bold=True, fg="blue")
 
+    # show known bad indicator
+    if signoff_pkg["known_bad"]:
+        formatted += click.style(" (known bad)", bold=True, fg="red")
+
     # show signed-off indicator if we're listing signed off packages
     status = signoff_status(signoff_pkg, options.username)
     if options.show_signed_off and status == "signed-off":
@@ -257,10 +262,14 @@ def format_signoff_long(signoff_pkg, local_pkg, options):
         signed_off = "Revoked"
     attributes.append(format_attr("Signed off", signed_off))
 
+    # bad status
+    if signoff_pkg["known_bad"]:
+        attributes.append(format_attr("Known bad", "Yes"))
+
     return "\n".join(attributes)
 
 
-def warn_if_outdated(signoff_pkg, local_pkg):
+def warn_if_outdated(signoff_pkg, local_pkg, color):
     """
     Echo a warning message if local and sign-off package versions differ.
     """
@@ -271,15 +280,30 @@ def warn_if_outdated(signoff_pkg, local_pkg):
             "({signoff_version})".format(
                 pkg=signoff_pkg["pkgbase"],
                 local_version=local_pkg.version,
-                signoff_version=signoff_pkg["version"]))
+                signoff_version=signoff_pkg["version"]), color=color)
 
 
-def confirm(text, *args, **kwargs):
+def warn_if_bad(signoff_pkg, color):
+    """
+    Echo a warning message if sign-off package is bad.
+    """
+    if signoff_pkg["known_bad"]:
+        click.echo(
+            click.style("warning:", fg="yellow", bold=True) +
+            " {pkg} is known to be bad".format(
+                pkg=signoff_pkg["pkgbase"]), color=color)
+
+
+def confirm(text, color, *args, **kwargs):
     """
     Wrapper around click.confirm that adds minor styling to the prompt.
     """
-    prefix = click.style(":: ", bold=True, fg="blue")
-    styled = click.style(text, bold=True)
+    if color:
+        prefix = click.style(":: ", bold=True, fg="blue")
+        styled = click.style(text, bold=True)
+    else:
+        prefix = ":: "
+        styled = click.unstyle(text)
     return click.confirm(prefix + styled, *args, **kwargs)
 
 
@@ -295,30 +319,31 @@ class Options:
 
 @click.command(context_settings={"help_option_names": ("-h", "--help")})
 @click.option("-s", "--signoff", "action", flag_value="signoff", help="sign "
-        "off packages")
+              "off packages")
 @click.option("-r", "--revoke", "action", flag_value="revoke", help="revoke "
-        "signed-off packages")
+              "signed-off packages")
 @click.option("-l", "--list", "action", flag_value="list", help="list "
-        "packages that can be signed off")
+              "packages that can be signed off")
 @click.option("-i", "--interactive", "action", flag_value="interactive",
-        help="interactively sign off packages")
+              help="interactively sign off packages")
 @click.option("-u", "--uninstalled", is_flag=True, help="include uninstalled "
-        "packages when listing")
+              "packages when listing")
 @click.option("-a", "--signed-off", is_flag=True, help="include signed-off "
-        "packages when listing")
+              "packages when listing")
 @click.option("-q", "--quiet", is_flag=True, help="be less verbose when "
-        "listing packages")
+              "listing packages")
 @click.option("--username", prompt=True, envvar="ARCHWEB_USERNAME",
-        help="ArchWeb username (ARCHWEB_USERNAME env. var.)")
+              help="ArchWeb username (ARCHWEB_USERNAME env. var.)")
 @click.option("--password", prompt=True, hide_input=True,
-        envvar="ARCHWEB_PASSWORD", help="ArchWeb password (ARCHWEB_PASSWORD "
-        "env. var.)")
+              envvar="ARCHWEB_PASSWORD", help="ArchWeb password (ARCHWEB_PASSWORD "
+              "env. var.)")
 @click.option("-b", "--db-path", type=click.Path(), default="/var/lib/pacman",
-        help="pacman database path")
+              help="pacman database path")
 @click.option("--noconfirm", is_flag=True, help="don't ask for confirmation")
+@click.option("--nocolor", is_flag=True, help="strip ANSI styles from output")
 @click.argument("package", nargs=-1)
 def main(action, uninstalled, signed_off, quiet, username, password, package,
-         db_path, noconfirm):
+         db_path, noconfirm, nocolor):
     """
     Interface with Arch Linux package signoffs.
     """
@@ -336,12 +361,24 @@ def main(action, uninstalled, signed_off, quiet, username, password, package,
         packages=set(package),
         db_path=db_path,
         username=username,
-        noconfirm=noconfirm)
+        noconfirm=noconfirm,
+        nocolor=nocolor)
 
     # initialize alpm handle and signoff session
-    alpm_handle = pyalpm.Handle("/", options.db_path)
+    try:
+        alpm_handle = pyalpm.Handle("/", options.db_path)
+    except pyalpm.error:
+        click.echo("error: could not read alpm database {}".format(options.db_path),
+                   err=True)
+        sys.exit(1)
+
     session = SignoffSession(options.username, password)
 
+    if options.nocolor or os.environ.get("TERM") == "dumb":
+        colorize = False
+    else:
+        colorize = True
+
     # fetch and filter signoff packages
     signoffs = list(list_signoffs(session, alpm_handle))
     packages = list(filter_signoffs(signoffs, options))
@@ -355,22 +392,28 @@ def main(action, uninstalled, signed_off, quiet, username, password, package,
 
     if action == "list":  # output packages and exit
         for signoff_pkg, local_pkg in packages:
-            click.echo(format_signoff(signoff_pkg, local_pkg, options))
+            click.echo(format_signoff(signoff_pkg, local_pkg, options), color=colorize)
             if not options.quiet:
                 click.echo()  # add a line between packages
     elif action == "signoff":  # sign-off packages
         for signoff_pkg, local_pkg in packages:
-            warn_if_outdated(signoff_pkg, local_pkg)
+            warn_if_outdated(signoff_pkg, local_pkg, colorize)
+            warn_if_bad(signoff_pkg, colorize)
         if options.noconfirm or confirm("Sign off {}?".format(
-                click.style(" ".join(pkgbases), bold=True))):
+                click.style(" ".join(pkgbases), bold=True)), colorize):
             for signoff_pkg, local_pkg in packages:
-                session.signoff_package(signoff_pkg)
-                click.echo("Signed off {}.".format(signoff_pkg["pkgbase"]))
+                try:
+                    session.signoff_package(signoff_pkg)
+                except requests.exceptions.HTTPError as e:
+                    click.echo("Could not sign off {} ({})".format(signoff_pkg["pkgbase"], e))
+                else:
+                    click.echo("Signed off {}.".format(signoff_pkg["pkgbase"]))
     elif action == "revoke":  # revoke sign-offs
         for signoff_pkg, local_pkg in packages:
-            warn_if_outdated(signoff_pkg, local_pkg)
+            warn_if_outdated(signoff_pkg, local_pkg, colorize)
+            warn_if_bad(signoff_pkg, colorize)
         if options.noconfirm or confirm("Revoke sign-off for {}?".format(
-                click.style(" ".join(pkgbases), bold=True))):
+                click.style(" ".join(pkgbases), bold=True)), colorize):
             for signoff_pkg, local_pkg in packages:
                 session.revoke_package(signoff_pkg)
                 click.echo("Revoked sign-off for {}.".format(
@@ -378,7 +421,8 @@ def main(action, uninstalled, signed_off, quiet, username, password, package,
     elif action == "interactive":  # interactively sign-off or revoke
         for signoff_pkg, local_pkg in packages:
             click.echo(format_signoff(signoff_pkg, local_pkg, options))
-            warn_if_outdated(signoff_pkg, local_pkg)
+            warn_if_outdated(signoff_pkg, local_pkg, colorize)
+            warn_if_bad(signoff_pkg, colorize)
             if not options.quiet:
                 click.echo()
 
@@ -393,7 +437,7 @@ def main(action, uninstalled, signed_off, quiet, username, password, package,
                 prompt = "Sign off {}?".format(pkgbase)
 
             # confirm and signoff/revoke
-            if confirm(prompt):
+            if confirm(prompt, colorize):
                 if signed_off:
                     session.revoke_package(signoff_pkg)
                     click.echo("Revoked sign-off for {}.".format(pkgbase))
